@@ -1,3 +1,8 @@
+//! Request/response types for the VTube Studio API.
+//!
+//! For a list of all request types, see the implementors for [`Request`].
+//! For the corresponding response types, see [`Response`].
+
 mod enumeration;
 mod envelope;
 mod error_id;
@@ -73,7 +78,9 @@ macro_rules! define_request_response_pairs {
         rust_name = $rust_name:ident,
         $(req_name = $req_name:literal,)?
         $(resp_name = $resp_name:literal,)?
+        $(#[doc = $req_doc:expr])+
         req = { $($req:tt)* },
+        $(#[doc = $resp_doc:expr])+
         resp = $(( $resp_inner:ident ))? $({ $($resp_fields:tt)* })?,
     },)*) => {
         paste! {
@@ -107,21 +114,30 @@ macro_rules! define_request_response_pairs {
         $(
             paste! {
                 #[allow(missing_docs)]
+                $(#[doc = $req_doc])+
+                ///
+                #[doc = concat!("This request returns [`", stringify!($rust_name), "Response`].")]
                 #[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
                 #[serde(rename_all = "camelCase")]
                 pub struct [<$rust_name Request>] { $($req)* }
 
                 impl Request for [<$rust_name Request>] {
                     type Response = [<$rust_name Response>];
+
+                    #[doc = concat!("[`RequestType::", stringify!($rust_name), "Request`]")]
                     const MESSAGE_TYPE: EnumString<RequestType> = EnumString::new(RequestType::[<$rust_name Request>]);
                 }
 
                 #[allow(missing_docs)]
+                $(#[doc = $resp_doc])+
+                ///
+                #[doc = concat!("This is the return value of [`", stringify!($rust_name), "Request`].")]
                 #[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
                 #[serde(rename_all = "camelCase")]
                 pub struct [<$rust_name Response>] $(($resp_inner);)? $({ $($resp_fields)* })?
 
                 impl Response for [<$rust_name Response>] {
+                    #[doc = concat!("[`ResponseType::", stringify!($rust_name), "Response`]")]
                     const MESSAGE_TYPE: EnumString<ResponseType> = EnumString::new(ResponseType::[<$rust_name Response>]);
                 }
             }
@@ -147,7 +163,9 @@ define_request_response_pairs!(
         rust_name = ApiState,
         req_name = "APIStateRequest",
         resp_name = "APIStateResponse",
+        /// Get the current state of the API.
         req = {},
+        /// The API state.
         resp = {
             pub active: bool,
             #[serde(rename = "vTubeStudioVersion")]
@@ -158,11 +176,14 @@ define_request_response_pairs!(
 
     {
         rust_name = AuthenticationToken,
+        /// Request an authentication token.
         req = {
             pub plugin_name: Cow<'static, str>,
             pub plugin_developer: Cow<'static, str>,
+            #[serde(skip_serializing_if = "Option::is_none")]
             pub plugin_icon: Option<Cow<'static, str>>,
         },
+        /// Authentication token response.
         resp = {
             pub authentication_token: String,
         },
@@ -170,11 +191,13 @@ define_request_response_pairs!(
 
     {
         rust_name = Authentication,
+        /// Authenticate with the API using a token.
         req = {
             pub plugin_name: Cow<'static, str>,
             pub plugin_developer: Cow<'static, str>,
             pub authentication_token: String,
         },
+        /// Whether the authentication request was successful.
         resp = {
             pub authenticated: bool,
             pub reason: String,
@@ -183,8 +206,11 @@ define_request_response_pairs!(
 
     {
         rust_name = Statistics,
+        /// Getting current VTS statistics.
         req = {},
+        /// Statistics about the VTube Studio session.
         resp = {
+            /// Uptime in milliseconds.
             pub uptime: i64,
             pub framerate: i32,
             #[serde(rename = "vTubeStudioVersion")]
@@ -202,7 +228,9 @@ define_request_response_pairs!(
         rust_name = VtsFolderInfo,
         req_name = "VTSFolderInfoRequest",
         resp_name = "VTSFolderInfoResponse",
+        /// Getting list of VTS folders.
         req = {},
+        /// Names of various folders in the `StreamingAssets` directory.
         resp = {
             pub models: String,
             pub backgrounds: String,
@@ -215,7 +243,9 @@ define_request_response_pairs!(
 
     {
         rust_name = CurrentModel,
+        /// Getting the currently loaded model.
         req = {},
+        /// Information about the current model.
         resp = {
             pub model_loaded: bool,
             pub model_name: String,
@@ -240,7 +270,9 @@ define_request_response_pairs!(
 
     {
         rust_name = AvailableModels,
+        /// Getting a list of available VTS models
         req = {},
+        /// List of available models.
         resp = {
             pub number_of_models: i32,
             pub available_models: Vec<Model>,
@@ -249,10 +281,12 @@ define_request_response_pairs!(
 
     {
         rust_name = ModelLoad,
+        /// Loading a VTS model by its ID.
         req = {
             #[serde(rename = "modelID")]
             pub model_id: String,
         },
+        /// Information about the loaded model ID.
         resp = {
             #[serde(rename = "modelID")]
             pub model_id: String,
@@ -262,23 +296,38 @@ define_request_response_pairs!(
 
     {
         rust_name = MoveModel,
+        /// Moving the currently loaded VTS model.
         req = {
+            /// How many seconds the animation should take. Maximum `2`.
             pub time_in_seconds: f64,
+            /// If `true`, apply movements relative to the model's current state.
             pub values_are_relative_to_model: bool,
+            /// Horizontal position. `-1` for left edge, `1` for right edge.
+            #[serde(skip_serializing_if = "Option::is_none")]
             pub position_x: Option<f64>,
+            /// Vertical position. `-1` for bottom edge, `1` for top edge.
+            #[serde(skip_serializing_if = "Option::is_none")]
             pub position_y: Option<f64>,
+            /// Rotation in degrees. Must be between `-360` and `360`.
+            #[serde(skip_serializing_if = "Option::is_none")]
             pub rotation: Option<f64>,
+            /// Size, between `-100` and `100`.
+            #[serde(skip_serializing_if = "Option::is_none")]
             pub size: Option<f64>,
         },
+        /// Empty response.
         resp = {},
     },
 
     {
         rust_name = HotkeysInCurrentModel,
+        /// Requesting list of hotkeys available in current or other VTS model.
         req = {
+            #[serde(skip_serializing_if = "Option::is_none")]
             #[serde(rename = "modelID")]
             pub model_id: Option<String>,
         },
+        /// Model info and list of hotkeys.
         resp = {
             pub model_loaded: bool,
             pub model_name: String,
@@ -290,10 +339,12 @@ define_request_response_pairs!(
 
     {
         rust_name = HotkeyTrigger,
+        /// Requesting execution of hotkeys.
         req = {
             #[serde(rename = "hotkeyID")]
             pub hotkey_id: String,
         },
+        /// The hotkey that was triggered.
         resp = {
             #[serde(rename = "hotkeyID")]
             pub hotkey_id: String,
@@ -302,7 +353,9 @@ define_request_response_pairs!(
 
     {
         rust_name = ArtMeshList,
+        /// Requesting list of ArtMeshes in current model.
         req = {},
+        /// List of ArtMeshes.
         resp = {
             pub model_loaded: bool,
             pub number_of_art_mesh_names: i32,
@@ -314,10 +367,12 @@ define_request_response_pairs!(
 
     {
         rust_name = ColorTint,
+        /// Tint ArtMeshes with color
         req = {
             pub color_tint: ColorTint,
             pub art_mesh_matcher: ArtMeshMatcher,
         },
+        /// Number of matched ArtMeshes.
         resp = {
             pub matched_art_meshes: i32,
         },
@@ -325,7 +380,9 @@ define_request_response_pairs!(
 
     {
         rust_name = SceneColorOverlayInfo,
+        /// Getting scene lighting overlay color.
         req = {},
+        /// Info about the color overlay.
         resp = {
             pub active: bool,
             pub items_included: bool,
@@ -347,7 +404,9 @@ define_request_response_pairs!(
 
     {
         rust_name = FaceFound,
+        /// Checking if face is currently found by tracker.
         req = {},
+        /// Whether the face was found.
         resp = {
             pub found: bool,
         },
@@ -355,7 +414,9 @@ define_request_response_pairs!(
 
     {
         rust_name = InputParameterList,
+        /// Requesting list of available tracking parameters.
         req = {},
+        /// List of available parameters.
         resp = {
             pub model_loaded: bool,
             pub model_name: String,
@@ -368,15 +429,19 @@ define_request_response_pairs!(
 
     {
         rust_name = ParameterValue,
+        /// Get the value for one specific parameter, default or custom.
         req = {
             pub name: String,
         },
+        /// The requested parameter.
         resp = (Parameter),
     },
 
     {
         rust_name = Live2DParameterList,
+        /// Get the value for all Live2D parameters in the current model.
         req = {},
+        /// Info about the current model and list of parameters.
         resp = {
             pub model_loaded: bool,
             pub model_name: String,
@@ -388,13 +453,16 @@ define_request_response_pairs!(
 
     {
         rust_name = ParameterCreation,
+        /// Adding new tracking parameters ("custom parameters").
         req = {
             pub parameter_name: String,
+            #[serde(skip_serializing_if = "Option::is_none")]
             pub explanation: Option<String>,
             pub min: f64,
             pub max: f64,
             pub default_value: f64,
         },
+        /// Name of the created parameter.
         resp = {
             pub parameter_name: String,
         },
@@ -402,9 +470,11 @@ define_request_response_pairs!(
 
     {
         rust_name = ParameterDeletion,
+        /// Delete custom parameters.
         req = {
             pub parameter_name: String,
         },
+        /// Name of the deleted parameter.
         resp = {
             pub parameter_name: String,
         },
@@ -412,21 +482,25 @@ define_request_response_pairs!(
 
     {
         rust_name = InjectParameterData,
+        /// Feeding in data for default or custom parameters.
         req = {
             pub parameter_values: Vec<ParameterValue>,
         },
+        /// Empty response.
         resp = {},
     },
 
 );
 
-#[allow(missing_docs)]
+/// Error returned by the VTube Studio API.
 #[derive(thiserror::Error, Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 #[error("APIError {error_id}: {message}")]
 pub struct ApiError {
+    /// The error ID.
     #[serde(rename = "errorID")]
     pub error_id: ErrorId,
+    /// A description of the error.
     pub message: String,
 }
 
@@ -441,6 +515,7 @@ impl ApiError {
     }
 }
 
+/// API Server Discovery (UDP).
 #[allow(missing_docs)]
 #[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
@@ -503,6 +578,7 @@ pub struct ColorTint {
     pub color_g: u8,
     pub color_b: u8,
     pub color_a: u8,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub mix_with_scene_lighting_color: Option<f64>,
     pub jeb_: bool,
 }
@@ -564,6 +640,7 @@ pub struct Parameter {
 pub struct ParameterValue {
     pub id: String,
     pub value: f64,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub weight: Option<f64>,
 }
 
